/*
 * _016_Rotary_Oled_PWM.c
 *
 * Created: 20-Dec-24 9:21:42 AM
 *  Author: Ajoy
 *
 * Rotary Encoder uses PB3 pin change interrupt and PB4
 * 10-Bit PWM on PB1 uses Timer1 overflow interrupt 
 * 10-Bit PWM is at ~8 kHz with fuse bit set for 8MHz no Divider/8
 * PB5 configured as ADC with threshold to detect Switch for faster counting
 * PBO and PB2 used as SCL and SDA for OLED
 */ 


#define  F_CPU 800000UL
#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include <stdint.h>
#include <stdbool.h>
#include "Aj_ssd1306_BitBang.c"

#define PWM10 PB1
#define EncoderA  PB3
#define EncoderB  PB4

//---------------10-Bit--------------
volatile int Dac = 128;
volatile int Cycle = 0;
volatile int PwmValue=255;

// Overflow interrupt
ISR (TIMER1_OVF_vect) {
	static int remain;
	if (Cycle == 0) remain = Dac;
if (remain >= 256) { OCR1A = 255; remain = remain - 256; }
	else { OCR1A = remain; remain = 0; }
	Cycle = (Cycle + 1) & 0x03;
}
//Rotary encoder ************************************

bool a0_previous;
bool c0_cleaned ;
int Count = 255;
bool CountUp, altCount = true, Countx10 = false;

//ADC for PB5 to determine switch position
uint16_t adc_high= 125; //value of Input Voltage in lower register

void ChangeValue (bool CountUp, bool Countx10);
// Called when encoder value changes
void adc_setup(); //Reset pin PB5 as ADC
// Setup PWM
void SetupPWM();



// Pin change interrupt service routine
ISR (PCINT0_vect) {
	//Grab state of input pins.
	bool a_current= (PINB & (1 << EncoderA )) ;
	bool b_current= (PINB & (1 << EncoderB )) ;
	//call ADC set x10 true/false
	Countx10 = false;
	//----Read ADC-----------
	ADCSRA |= (1<<ADSC);//start conversion
	while (ADCSRA & (1 << ADSC) ){}; // wait for the conversion to finish
		adc_high= ADCH;
		if (adc_high < 250){
			Countx10 = true;
		}
		if (a_current != a0_previous) {  // A changed
		a0_previous = a_current;
		if (b_current != c0_cleaned) {
			c0_cleaned = b_current;
			CountUp = ( a_current == b_current);//== returns true or false
			ChangeValue(CountUp, Countx10);
		}
  }   
}

int main() {
	//---Initialize OLED-------------------
	ssd1306_init();
	initFrame();
	ssd1306_fillscreen(0x00);
	_delay_ms(1000);
	
	ssd1306_switchRenderFrame();
    //---------Setup ADC-------------------
	adc_setup();
    //---------Initialize I/O---------------
    // Set port B pins 3,4 to In Default is input (0)
	// Pull up provided in HW
	//--------PWM---------------
	// PWM10 pin as output
	DDRB |= _BV(PWM10);
	SetupPWM();
	
	//----Encoder Interrupt----------
	PCMSK = 1 << EncoderA;        // Configure pin change interrupt on A
	GIMSK = 1 << PCIE;            // Enable interrupt
	GIFR =  1 << PCIF;             // Clear interrupt flag
	
	sei();
	
	while(1){
		
		display_V(Count);
		_delay_ms(20) ;
		ssd1306_switchFrame();
		//cli(); Dac = Count; sei();		
		//_delay_ms(20) ;
		TIMSK &=~ (1<<TOIE1);//disable timer interrupt
		Dac = Count;
		TIMSK |= (1<<TOIE1); //enable timer interrupt			
	}
}

void ChangeValue (bool CountUp, bool Countx10) {
	if (altCount==true){
		if (CountUp == true) {			
			if (Countx10 == true){
				Count = Count+ 50;
			}
			else {
				Count = Count++;
			}				
		}
		else if (CountUp == false) {			
			if (Countx10 == true){
				Count = Count- 50;
			}
			else {
				Count = Count--;
			}
		}
	Count= Count & 0x03ff;
	}	
	altCount= ! altCount;
}

void adc_setup()
{	ADMUX |=(1<<ADLAR);// High Byte 8-Bit rest  default // PB5 input, VCC ref, ADCH/L out
	ADCSRA = (1<<ADEN)|(1<<ADPS2); //Enable ADC module, Clock/16
	//ADCSRB default free running mode
	//Free running mode, start conversion ADCSRA |=1<<ADSC);
	//bit is reset on conversion complete
}	

void SetupPWM(){
	//CKSEL
	OSCCAL = 181; //to inc the freq	
	// Timer/Counter1 doing PWM on OC1A (PB1)	
	TCCR1 = 1<<PWM1A | 1<<COM1A1 | 1<<CS10;
	//COM1A1 COM1A0  1 0 gives a single PWM
	// Clk no divider, Toggle the OC1A output line
	//Pulse Width Modulator A Enable	
	TIMSK |=1<<TOIE1;
	//Overflow Interrupt enable	
	//PwmValue=255;
	//cli(); Dac = value; sei();
	
	
}